package io.syndesis.server.controller.integration.camelk.crd;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import com.google.errorprone.annotations.Var;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import javax.annotation.CheckReturnValue;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link IntegrationSpec}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new IntegrationSpec.Builder()}.
 */
@Generated(from = "IntegrationSpec", generator = "Immutables")
@SuppressWarnings({"ArrayEquals", "ArrayHashCode", "ArrayToString", "all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
@CheckReturnValue
public final class ImmutableIntegrationSpec
    implements IntegrationSpec {
  private final @Nullable Integer replicas;
  private final ImmutableList<SourceSpec> sources;
  private final ImmutableList<ResourceSpec> resources;
  private final @Nullable String context;
  private final ImmutableList<String> dependencies;
  private final @Nullable String profile;
  private final ImmutableMap<String, IntegrationTraitSpec> traits;
  private final ImmutableList<ConfigurationSpec> configuration;
  private final ImmutableList<String> repositories;
  private final @Nullable String serviceAccountName;

  private ImmutableIntegrationSpec(ImmutableIntegrationSpec.Builder builder) {
    this.replicas = builder.replicas;
    this.context = builder.context;
    this.profile = builder.profile;
    this.serviceAccountName = builder.serviceAccountName;
    if (builder.sourcesIsSet()) {
      initShim.sources(builder.sources.build());
    }
    if (builder.resourcesIsSet()) {
      initShim.resources(builder.resources.build());
    }
    if (builder.dependenciesIsSet()) {
      initShim.dependencies(builder.dependencies.build());
    }
    if (builder.traitsIsSet()) {
      initShim.traits(builder.traits.build());
    }
    if (builder.configurationIsSet()) {
      initShim.configuration(builder.configuration.build());
    }
    if (builder.repositoriesIsSet()) {
      initShim.repositories(builder.repositories.build());
    }
    this.sources = initShim.getSources();
    this.resources = initShim.getResources();
    this.dependencies = initShim.getDependencies();
    this.traits = initShim.getTraits();
    this.configuration = initShim.getConfiguration();
    this.repositories = initShim.getRepositories();
    this.initShim = null;
  }

  private ImmutableIntegrationSpec(
      @Nullable Integer replicas,
      ImmutableList<SourceSpec> sources,
      ImmutableList<ResourceSpec> resources,
      @Nullable String context,
      ImmutableList<String> dependencies,
      @Nullable String profile,
      ImmutableMap<String, IntegrationTraitSpec> traits,
      ImmutableList<ConfigurationSpec> configuration,
      ImmutableList<String> repositories,
      @Nullable String serviceAccountName) {
    this.replicas = replicas;
    this.sources = sources;
    this.resources = resources;
    this.context = context;
    this.dependencies = dependencies;
    this.profile = profile;
    this.traits = traits;
    this.configuration = configuration;
    this.repositories = repositories;
    this.serviceAccountName = serviceAccountName;
    this.initShim = null;
  }

  private static final byte STAGE_INITIALIZING = -1;
  private static final byte STAGE_UNINITIALIZED = 0;
  private static final byte STAGE_INITIALIZED = 1;
  @SuppressWarnings("Immutable")
  private transient volatile InitShim initShim = new InitShim();

  @Generated(from = "IntegrationSpec", generator = "Immutables")
  private final class InitShim {
    private byte sourcesBuildStage = STAGE_UNINITIALIZED;
    private ImmutableList<SourceSpec> sources;

    ImmutableList<SourceSpec> getSources() {
      if (sourcesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (sourcesBuildStage == STAGE_UNINITIALIZED) {
        sourcesBuildStage = STAGE_INITIALIZING;
        this.sources = ImmutableList.copyOf(getSourcesInitialize());
        sourcesBuildStage = STAGE_INITIALIZED;
      }
      return this.sources;
    }

    void sources(ImmutableList<SourceSpec> sources) {
      this.sources = sources;
      sourcesBuildStage = STAGE_INITIALIZED;
    }

    private byte resourcesBuildStage = STAGE_UNINITIALIZED;
    private ImmutableList<ResourceSpec> resources;

    ImmutableList<ResourceSpec> getResources() {
      if (resourcesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (resourcesBuildStage == STAGE_UNINITIALIZED) {
        resourcesBuildStage = STAGE_INITIALIZING;
        this.resources = ImmutableList.copyOf(getResourcesInitialize());
        resourcesBuildStage = STAGE_INITIALIZED;
      }
      return this.resources;
    }

    void resources(ImmutableList<ResourceSpec> resources) {
      this.resources = resources;
      resourcesBuildStage = STAGE_INITIALIZED;
    }

    private byte dependenciesBuildStage = STAGE_UNINITIALIZED;
    private ImmutableList<String> dependencies;

    ImmutableList<String> getDependencies() {
      if (dependenciesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (dependenciesBuildStage == STAGE_UNINITIALIZED) {
        dependenciesBuildStage = STAGE_INITIALIZING;
        this.dependencies = ImmutableList.copyOf(getDependenciesInitialize());
        dependenciesBuildStage = STAGE_INITIALIZED;
      }
      return this.dependencies;
    }

    void dependencies(ImmutableList<String> dependencies) {
      this.dependencies = dependencies;
      dependenciesBuildStage = STAGE_INITIALIZED;
    }

    private byte traitsBuildStage = STAGE_UNINITIALIZED;
    private ImmutableMap<String, IntegrationTraitSpec> traits;

    ImmutableMap<String, IntegrationTraitSpec> getTraits() {
      if (traitsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (traitsBuildStage == STAGE_UNINITIALIZED) {
        traitsBuildStage = STAGE_INITIALIZING;
        this.traits = ImmutableMap.copyOf(getTraitsInitialize());
        traitsBuildStage = STAGE_INITIALIZED;
      }
      return this.traits;
    }

    void traits(ImmutableMap<String, IntegrationTraitSpec> traits) {
      this.traits = traits;
      traitsBuildStage = STAGE_INITIALIZED;
    }

    private byte configurationBuildStage = STAGE_UNINITIALIZED;
    private ImmutableList<ConfigurationSpec> configuration;

    ImmutableList<ConfigurationSpec> getConfiguration() {
      if (configurationBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (configurationBuildStage == STAGE_UNINITIALIZED) {
        configurationBuildStage = STAGE_INITIALIZING;
        this.configuration = ImmutableList.copyOf(getConfigurationInitialize());
        configurationBuildStage = STAGE_INITIALIZED;
      }
      return this.configuration;
    }

    void configuration(ImmutableList<ConfigurationSpec> configuration) {
      this.configuration = configuration;
      configurationBuildStage = STAGE_INITIALIZED;
    }

    private byte repositoriesBuildStage = STAGE_UNINITIALIZED;
    private ImmutableList<String> repositories;

    ImmutableList<String> getRepositories() {
      if (repositoriesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (repositoriesBuildStage == STAGE_UNINITIALIZED) {
        repositoriesBuildStage = STAGE_INITIALIZING;
        this.repositories = ImmutableList.copyOf(getRepositoriesInitialize());
        repositoriesBuildStage = STAGE_INITIALIZED;
      }
      return this.repositories;
    }

    void repositories(ImmutableList<String> repositories) {
      this.repositories = repositories;
      repositoriesBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      List<String> attributes = new ArrayList<>();
      if (sourcesBuildStage == STAGE_INITIALIZING) attributes.add("sources");
      if (resourcesBuildStage == STAGE_INITIALIZING) attributes.add("resources");
      if (dependenciesBuildStage == STAGE_INITIALIZING) attributes.add("dependencies");
      if (traitsBuildStage == STAGE_INITIALIZING) attributes.add("traits");
      if (configurationBuildStage == STAGE_INITIALIZING) attributes.add("configuration");
      if (repositoriesBuildStage == STAGE_INITIALIZING) attributes.add("repositories");
      return "Cannot build IntegrationSpec, attribute initializers form cycle " + attributes;
    }
  }

  private List<SourceSpec> getSourcesInitialize() {
    return IntegrationSpec.super.getSources();
  }

  private List<ResourceSpec> getResourcesInitialize() {
    return IntegrationSpec.super.getResources();
  }

  private List<String> getDependenciesInitialize() {
    return IntegrationSpec.super.getDependencies();
  }

  private Map<String, IntegrationTraitSpec> getTraitsInitialize() {
    return IntegrationSpec.super.getTraits();
  }

  private List<ConfigurationSpec> getConfigurationInitialize() {
    return IntegrationSpec.super.getConfiguration();
  }

  private List<String> getRepositoriesInitialize() {
    return IntegrationSpec.super.getRepositories();
  }

  /**
   * @return The value of the {@code replicas} attribute
   */
  @JsonProperty("replicas")
  @Override
  public @Nullable Integer getReplicas() {
    return replicas;
  }

  /**
   * @return The value of the {@code sources} attribute
   */
  @JsonProperty("sources")
  @Override
  public ImmutableList<SourceSpec> getSources() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getSources()
        : this.sources;
  }

  /**
   * @return The value of the {@code resources} attribute
   */
  @JsonProperty("resources")
  @Override
  public ImmutableList<ResourceSpec> getResources() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getResources()
        : this.resources;
  }

  /**
   * @return The value of the {@code context} attribute
   */
  @JsonProperty("context")
  @Override
  public @Nullable String getContext() {
    return context;
  }

  /**
   * @return The value of the {@code dependencies} attribute
   */
  @JsonProperty("dependencies")
  @Override
  public ImmutableList<String> getDependencies() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getDependencies()
        : this.dependencies;
  }

  /**
   * @return The value of the {@code profile} attribute
   */
  @JsonProperty("profile")
  @Override
  public @Nullable String getProfile() {
    return profile;
  }

  /**
   * @return The value of the {@code traits} attribute
   */
  @JsonProperty("traits")
  @Override
  public ImmutableMap<String, IntegrationTraitSpec> getTraits() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getTraits()
        : this.traits;
  }

  /**
   * @return The value of the {@code configuration} attribute
   */
  @JsonProperty("configuration")
  @Override
  public ImmutableList<ConfigurationSpec> getConfiguration() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getConfiguration()
        : this.configuration;
  }

  /**
   * @return The value of the {@code repositories} attribute
   */
  @JsonProperty("repositories")
  @Override
  public ImmutableList<String> getRepositories() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getRepositories()
        : this.repositories;
  }

  /**
   * @return The value of the {@code serviceAccountName} attribute
   */
  @JsonProperty("serviceAccountName")
  @Override
  public @Nullable String getServiceAccountName() {
    return serviceAccountName;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationSpec#getReplicas() replicas} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for replicas (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationSpec withReplicas(@Nullable Integer value) {
    if (Objects.equals(this.replicas, value)) return this;
    return new ImmutableIntegrationSpec(
        value,
        this.sources,
        this.resources,
        this.context,
        this.dependencies,
        this.profile,
        this.traits,
        this.configuration,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationSpec#getSources() sources}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withSources(SourceSpec... elements) {
    ImmutableList<SourceSpec> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationSpec(
        this.replicas,
        newValue,
        this.resources,
        this.context,
        this.dependencies,
        this.profile,
        this.traits,
        this.configuration,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationSpec#getSources() sources}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of sources elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withSources(Iterable<? extends SourceSpec> elements) {
    if (this.sources == elements) return this;
    ImmutableList<SourceSpec> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationSpec(
        this.replicas,
        newValue,
        this.resources,
        this.context,
        this.dependencies,
        this.profile,
        this.traits,
        this.configuration,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationSpec#getResources() resources}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withResources(ResourceSpec... elements) {
    ImmutableList<ResourceSpec> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        newValue,
        this.context,
        this.dependencies,
        this.profile,
        this.traits,
        this.configuration,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationSpec#getResources() resources}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of resources elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withResources(Iterable<? extends ResourceSpec> elements) {
    if (this.resources == elements) return this;
    ImmutableList<ResourceSpec> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        newValue,
        this.context,
        this.dependencies,
        this.profile,
        this.traits,
        this.configuration,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationSpec#getContext() context} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for context (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationSpec withContext(@Nullable String value) {
    if (Objects.equals(this.context, value)) return this;
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        this.resources,
        value,
        this.dependencies,
        this.profile,
        this.traits,
        this.configuration,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationSpec#getDependencies() dependencies}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withDependencies(String... elements) {
    ImmutableList<String> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        this.resources,
        this.context,
        newValue,
        this.profile,
        this.traits,
        this.configuration,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationSpec#getDependencies() dependencies}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of dependencies elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withDependencies(Iterable<String> elements) {
    if (this.dependencies == elements) return this;
    ImmutableList<String> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        this.resources,
        this.context,
        newValue,
        this.profile,
        this.traits,
        this.configuration,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationSpec#getProfile() profile} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for profile (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationSpec withProfile(@Nullable String value) {
    if (Objects.equals(this.profile, value)) return this;
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        this.resources,
        this.context,
        this.dependencies,
        value,
        this.traits,
        this.configuration,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object by replacing the {@link IntegrationSpec#getTraits() traits} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the traits map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withTraits(Map<String, ? extends IntegrationTraitSpec> entries) {
    if (this.traits == entries) return this;
    ImmutableMap<String, IntegrationTraitSpec> newValue = ImmutableMap.copyOf(entries);
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        this.resources,
        this.context,
        this.dependencies,
        this.profile,
        newValue,
        this.configuration,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationSpec#getConfiguration() configuration}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withConfiguration(ConfigurationSpec... elements) {
    ImmutableList<ConfigurationSpec> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        this.resources,
        this.context,
        this.dependencies,
        this.profile,
        this.traits,
        newValue,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationSpec#getConfiguration() configuration}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of configuration elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withConfiguration(Iterable<? extends ConfigurationSpec> elements) {
    if (this.configuration == elements) return this;
    ImmutableList<ConfigurationSpec> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        this.resources,
        this.context,
        this.dependencies,
        this.profile,
        this.traits,
        newValue,
        this.repositories,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationSpec#getRepositories() repositories}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withRepositories(String... elements) {
    ImmutableList<String> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        this.resources,
        this.context,
        this.dependencies,
        this.profile,
        this.traits,
        this.configuration,
        newValue,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationSpec#getRepositories() repositories}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of repositories elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationSpec withRepositories(Iterable<String> elements) {
    if (this.repositories == elements) return this;
    ImmutableList<String> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        this.resources,
        this.context,
        this.dependencies,
        this.profile,
        this.traits,
        this.configuration,
        newValue,
        this.serviceAccountName);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationSpec#getServiceAccountName() serviceAccountName} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for serviceAccountName (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationSpec withServiceAccountName(@Nullable String value) {
    if (Objects.equals(this.serviceAccountName, value)) return this;
    return new ImmutableIntegrationSpec(
        this.replicas,
        this.sources,
        this.resources,
        this.context,
        this.dependencies,
        this.profile,
        this.traits,
        this.configuration,
        this.repositories,
        value);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableIntegrationSpec} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableIntegrationSpec
        && equalTo((ImmutableIntegrationSpec) another);
  }

  private boolean equalTo(ImmutableIntegrationSpec another) {
    return Objects.equals(replicas, another.replicas)
        && sources.equals(another.sources)
        && resources.equals(another.resources)
        && Objects.equals(context, another.context)
        && dependencies.equals(another.dependencies)
        && Objects.equals(profile, another.profile)
        && traits.equals(another.traits)
        && configuration.equals(another.configuration)
        && repositories.equals(another.repositories)
        && Objects.equals(serviceAccountName, another.serviceAccountName);
  }

  /**
   * Computes a hash code from attributes: {@code replicas}, {@code sources}, {@code resources}, {@code context}, {@code dependencies}, {@code profile}, {@code traits}, {@code configuration}, {@code repositories}, {@code serviceAccountName}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    @Var int h = 5381;
    h += (h << 5) + Objects.hashCode(replicas);
    h += (h << 5) + sources.hashCode();
    h += (h << 5) + resources.hashCode();
    h += (h << 5) + Objects.hashCode(context);
    h += (h << 5) + dependencies.hashCode();
    h += (h << 5) + Objects.hashCode(profile);
    h += (h << 5) + traits.hashCode();
    h += (h << 5) + configuration.hashCode();
    h += (h << 5) + repositories.hashCode();
    h += (h << 5) + Objects.hashCode(serviceAccountName);
    return h;
  }

  /**
   * Prints the immutable value {@code IntegrationSpec} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("IntegrationSpec")
        .omitNullValues()
        .add("replicas", replicas)
        .add("sources", sources)
        .add("resources", resources)
        .add("context", context)
        .add("dependencies", dependencies)
        .add("profile", profile)
        .add("traits", traits)
        .add("configuration", configuration)
        .add("repositories", repositories)
        .add("serviceAccountName", serviceAccountName)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link IntegrationSpec} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable IntegrationSpec instance
   */
  public static ImmutableIntegrationSpec copyOf(IntegrationSpec instance) {
    if (instance instanceof ImmutableIntegrationSpec) {
      return (ImmutableIntegrationSpec) instance;
    }
    return new IntegrationSpec.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableIntegrationSpec ImmutableIntegrationSpec}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "IntegrationSpec", generator = "Immutables")
  @NotThreadSafe
  @JsonInclude(JsonInclude.Include.NON_NULL)
  public static class Builder {
    private static final long OPT_BIT_SOURCES = 0x1L;
    private static final long OPT_BIT_RESOURCES = 0x2L;
    private static final long OPT_BIT_DEPENDENCIES = 0x4L;
    private static final long OPT_BIT_TRAITS = 0x8L;
    private static final long OPT_BIT_CONFIGURATION = 0x10L;
    private static final long OPT_BIT_REPOSITORIES = 0x20L;
    private long optBits;

    private @Nullable Integer replicas;
    private ImmutableList.Builder<SourceSpec> sources = ImmutableList.builder();
    private ImmutableList.Builder<ResourceSpec> resources = ImmutableList.builder();
    private @Nullable String context;
    private ImmutableList.Builder<String> dependencies = ImmutableList.builder();
    private @Nullable String profile;
    private ImmutableMap.Builder<String, IntegrationTraitSpec> traits = ImmutableMap.builder();
    private ImmutableList.Builder<ConfigurationSpec> configuration = ImmutableList.builder();
    private ImmutableList.Builder<String> repositories = ImmutableList.builder();
    private @Nullable String serviceAccountName;

    /**
     * Creates a builder for {@link ImmutableIntegrationSpec ImmutableIntegrationSpec} instances.
     * <pre>
     * new IntegrationSpec.Builder()
     *    .replicas(Integer | null) // nullable {@link IntegrationSpec#getReplicas() replicas}
     *    .addSources|addAllSources(io.syndesis.server.controller.integration.camelk.crd.SourceSpec) // {@link IntegrationSpec#getSources() sources} elements
     *    .addResources|addAllResources(io.syndesis.server.controller.integration.camelk.crd.ResourceSpec) // {@link IntegrationSpec#getResources() resources} elements
     *    .context(String | null) // nullable {@link IntegrationSpec#getContext() context}
     *    .addDependencies|addAllDependencies(String) // {@link IntegrationSpec#getDependencies() dependencies} elements
     *    .profile(String | null) // nullable {@link IntegrationSpec#getProfile() profile}
     *    .putTraits|putAllTraits(String =&gt; io.syndesis.server.controller.integration.camelk.crd.IntegrationTraitSpec) // {@link IntegrationSpec#getTraits() traits} mappings
     *    .addConfiguration|addAllConfiguration(io.syndesis.server.controller.integration.camelk.crd.ConfigurationSpec) // {@link IntegrationSpec#getConfiguration() configuration} elements
     *    .addRepositories|addAllRepositories(String) // {@link IntegrationSpec#getRepositories() repositories} elements
     *    .serviceAccountName(String | null) // nullable {@link IntegrationSpec#getServiceAccountName() serviceAccountName}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof IntegrationSpec.Builder)) {
        throw new UnsupportedOperationException("Use: new IntegrationSpec.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code IntegrationSpec} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder from(IntegrationSpec instance) {
      Objects.requireNonNull(instance, "instance");
      @Nullable Integer replicasValue = instance.getReplicas();
      if (replicasValue != null) {
        replicas(replicasValue);
      }
      addAllSources(instance.getSources());
      addAllResources(instance.getResources());
      @Nullable String contextValue = instance.getContext();
      if (contextValue != null) {
        context(contextValue);
      }
      addAllDependencies(instance.getDependencies());
      @Nullable String profileValue = instance.getProfile();
      if (profileValue != null) {
        profile(profileValue);
      }
      putAllTraits(instance.getTraits());
      addAllConfiguration(instance.getConfiguration());
      addAllRepositories(instance.getRepositories());
      @Nullable String serviceAccountNameValue = instance.getServiceAccountName();
      if (serviceAccountNameValue != null) {
        serviceAccountName(serviceAccountNameValue);
      }
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationSpec#getReplicas() replicas} attribute.
     * @param replicas The value for replicas (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("replicas")
    public final IntegrationSpec.Builder replicas(@Nullable Integer replicas) {
      this.replicas = replicas;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationSpec#getSources() sources} list.
     * @param element A sources element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addSources(SourceSpec element) {
      this.sources.add(element);
      optBits |= OPT_BIT_SOURCES;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationSpec#getSources() sources} list.
     * @param elements An array of sources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addSources(SourceSpec... elements) {
      this.sources.add(elements);
      optBits |= OPT_BIT_SOURCES;
      return (IntegrationSpec.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link IntegrationSpec#getSources() sources} list.
     * @param elements An iterable of sources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("sources")
    public final IntegrationSpec.Builder sources(Iterable<? extends SourceSpec> elements) {
      this.sources = ImmutableList.builder();
      return addAllSources(elements);
    }

    /**
     * Adds elements to {@link IntegrationSpec#getSources() sources} list.
     * @param elements An iterable of sources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addAllSources(Iterable<? extends SourceSpec> elements) {
      this.sources.addAll(elements);
      optBits |= OPT_BIT_SOURCES;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationSpec#getResources() resources} list.
     * @param element A resources element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addResources(ResourceSpec element) {
      this.resources.add(element);
      optBits |= OPT_BIT_RESOURCES;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationSpec#getResources() resources} list.
     * @param elements An array of resources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addResources(ResourceSpec... elements) {
      this.resources.add(elements);
      optBits |= OPT_BIT_RESOURCES;
      return (IntegrationSpec.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link IntegrationSpec#getResources() resources} list.
     * @param elements An iterable of resources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("resources")
    public final IntegrationSpec.Builder resources(Iterable<? extends ResourceSpec> elements) {
      this.resources = ImmutableList.builder();
      return addAllResources(elements);
    }

    /**
     * Adds elements to {@link IntegrationSpec#getResources() resources} list.
     * @param elements An iterable of resources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addAllResources(Iterable<? extends ResourceSpec> elements) {
      this.resources.addAll(elements);
      optBits |= OPT_BIT_RESOURCES;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationSpec#getContext() context} attribute.
     * @param context The value for context (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("context")
    public final IntegrationSpec.Builder context(@Nullable String context) {
      this.context = context;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationSpec#getDependencies() dependencies} list.
     * @param element A dependencies element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addDependencies(String element) {
      this.dependencies.add(element);
      optBits |= OPT_BIT_DEPENDENCIES;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationSpec#getDependencies() dependencies} list.
     * @param elements An array of dependencies elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addDependencies(String... elements) {
      this.dependencies.add(elements);
      optBits |= OPT_BIT_DEPENDENCIES;
      return (IntegrationSpec.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link IntegrationSpec#getDependencies() dependencies} list.
     * @param elements An iterable of dependencies elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("dependencies")
    public final IntegrationSpec.Builder dependencies(Iterable<String> elements) {
      this.dependencies = ImmutableList.builder();
      return addAllDependencies(elements);
    }

    /**
     * Adds elements to {@link IntegrationSpec#getDependencies() dependencies} list.
     * @param elements An iterable of dependencies elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addAllDependencies(Iterable<String> elements) {
      this.dependencies.addAll(elements);
      optBits |= OPT_BIT_DEPENDENCIES;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationSpec#getProfile() profile} attribute.
     * @param profile The value for profile (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("profile")
    public final IntegrationSpec.Builder profile(@Nullable String profile) {
      this.profile = profile;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Put one entry to the {@link IntegrationSpec#getTraits() traits} map.
     * @param key The key in the traits map
     * @param value The associated value in the traits map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder putTraits(String key, IntegrationTraitSpec value) {
      this.traits.put(key, value);
      optBits |= OPT_BIT_TRAITS;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Put one entry to the {@link IntegrationSpec#getTraits() traits} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder putTraits(Map.Entry<String, ? extends IntegrationTraitSpec> entry) {
      this.traits.put(entry);
      optBits |= OPT_BIT_TRAITS;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link IntegrationSpec#getTraits() traits} map. Nulls are not permitted
     * @param entries The entries that will be added to the traits map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("traits")
    public final IntegrationSpec.Builder traits(Map<String, ? extends IntegrationTraitSpec> entries) {
      this.traits = ImmutableMap.builder();
      optBits |= OPT_BIT_TRAITS;
      return putAllTraits(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link IntegrationSpec#getTraits() traits} map. Nulls are not permitted
     * @param entries The entries that will be added to the traits map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder putAllTraits(Map<String, ? extends IntegrationTraitSpec> entries) {
      this.traits.putAll(entries);
      optBits |= OPT_BIT_TRAITS;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationSpec#getConfiguration() configuration} list.
     * @param element A configuration element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addConfiguration(ConfigurationSpec element) {
      this.configuration.add(element);
      optBits |= OPT_BIT_CONFIGURATION;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationSpec#getConfiguration() configuration} list.
     * @param elements An array of configuration elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addConfiguration(ConfigurationSpec... elements) {
      this.configuration.add(elements);
      optBits |= OPT_BIT_CONFIGURATION;
      return (IntegrationSpec.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link IntegrationSpec#getConfiguration() configuration} list.
     * @param elements An iterable of configuration elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("configuration")
    public final IntegrationSpec.Builder configuration(Iterable<? extends ConfigurationSpec> elements) {
      this.configuration = ImmutableList.builder();
      return addAllConfiguration(elements);
    }

    /**
     * Adds elements to {@link IntegrationSpec#getConfiguration() configuration} list.
     * @param elements An iterable of configuration elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addAllConfiguration(Iterable<? extends ConfigurationSpec> elements) {
      this.configuration.addAll(elements);
      optBits |= OPT_BIT_CONFIGURATION;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationSpec#getRepositories() repositories} list.
     * @param element A repositories element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addRepositories(String element) {
      this.repositories.add(element);
      optBits |= OPT_BIT_REPOSITORIES;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationSpec#getRepositories() repositories} list.
     * @param elements An array of repositories elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addRepositories(String... elements) {
      this.repositories.add(elements);
      optBits |= OPT_BIT_REPOSITORIES;
      return (IntegrationSpec.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link IntegrationSpec#getRepositories() repositories} list.
     * @param elements An iterable of repositories elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("repositories")
    public final IntegrationSpec.Builder repositories(Iterable<String> elements) {
      this.repositories = ImmutableList.builder();
      return addAllRepositories(elements);
    }

    /**
     * Adds elements to {@link IntegrationSpec#getRepositories() repositories} list.
     * @param elements An iterable of repositories elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationSpec.Builder addAllRepositories(Iterable<String> elements) {
      this.repositories.addAll(elements);
      optBits |= OPT_BIT_REPOSITORIES;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationSpec#getServiceAccountName() serviceAccountName} attribute.
     * @param serviceAccountName The value for serviceAccountName (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("serviceAccountName")
    public final IntegrationSpec.Builder serviceAccountName(@Nullable String serviceAccountName) {
      this.serviceAccountName = serviceAccountName;
      return (IntegrationSpec.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableIntegrationSpec ImmutableIntegrationSpec}.
     * @return An immutable instance of IntegrationSpec
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableIntegrationSpec build() {
      return new ImmutableIntegrationSpec(this);
    }

    private boolean sourcesIsSet() {
      return (optBits & OPT_BIT_SOURCES) != 0;
    }

    private boolean resourcesIsSet() {
      return (optBits & OPT_BIT_RESOURCES) != 0;
    }

    private boolean dependenciesIsSet() {
      return (optBits & OPT_BIT_DEPENDENCIES) != 0;
    }

    private boolean traitsIsSet() {
      return (optBits & OPT_BIT_TRAITS) != 0;
    }

    private boolean configurationIsSet() {
      return (optBits & OPT_BIT_CONFIGURATION) != 0;
    }

    private boolean repositoriesIsSet() {
      return (optBits & OPT_BIT_REPOSITORIES) != 0;
    }
  }
}
