package io.syndesis.server.controller.integration.camelk.crd;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableList;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import com.google.errorprone.annotations.Var;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Objects;
import javax.annotation.CheckReturnValue;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link IntegrationStatus}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new IntegrationStatus.Builder()}.
 */
@Generated(from = "IntegrationStatus", generator = "Immutables")
@SuppressWarnings({"ArrayEquals", "ArrayHashCode", "ArrayToString", "all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
@CheckReturnValue
public final class ImmutableIntegrationStatus
    implements IntegrationStatus {
  private final @Nullable String phase;
  private final @Nullable String digest;
  private final @Nullable String image;
  private final ImmutableList<String> dependencies;
  private final @Nullable String context;
  private final ImmutableList<SourceSpec> generatedSources;
  private final @Nullable Failure failure;
  private final @Nullable String camelVersion;
  private final @Nullable String runtimeVersion;

  private ImmutableIntegrationStatus(
      @Nullable String phase,
      @Nullable String digest,
      @Nullable String image,
      ImmutableList<String> dependencies,
      @Nullable String context,
      ImmutableList<SourceSpec> generatedSources,
      @Nullable Failure failure,
      @Nullable String camelVersion,
      @Nullable String runtimeVersion) {
    this.phase = phase;
    this.digest = digest;
    this.image = image;
    this.dependencies = dependencies;
    this.context = context;
    this.generatedSources = generatedSources;
    this.failure = failure;
    this.camelVersion = camelVersion;
    this.runtimeVersion = runtimeVersion;
  }

  /**
   * @return The value of the {@code phase} attribute
   */
  @JsonProperty("phase")
  @Override
  public @Nullable String getPhase() {
    return phase;
  }

  /**
   * @return The value of the {@code digest} attribute
   */
  @JsonProperty("digest")
  @Override
  public @Nullable String getDigest() {
    return digest;
  }

  /**
   * @return The value of the {@code image} attribute
   */
  @JsonProperty("image")
  @Override
  public @Nullable String getImage() {
    return image;
  }

  /**
   * @return The value of the {@code dependencies} attribute
   */
  @JsonProperty("dependencies")
  @Override
  public ImmutableList<String> getDependencies() {
    return dependencies;
  }

  /**
   * @return The value of the {@code context} attribute
   */
  @JsonProperty("context")
  @Override
  public @Nullable String getContext() {
    return context;
  }

  /**
   * @return The value of the {@code generatedSources} attribute
   */
  @JsonProperty("generatedSources")
  @Override
  public ImmutableList<SourceSpec> getGeneratedSources() {
    return generatedSources;
  }

  /**
   * @return The value of the {@code failure} attribute
   */
  @JsonProperty("failure")
  @Override
  public @Nullable Failure getFailure() {
    return failure;
  }

  /**
   * @return The value of the {@code camelVersion} attribute
   */
  @JsonProperty("camelVersion")
  @Override
  public @Nullable String getCamelVersion() {
    return camelVersion;
  }

  /**
   * @return The value of the {@code runtimeVersion} attribute
   */
  @JsonProperty("runtimeVersion")
  @Override
  public @Nullable String getRuntimeVersion() {
    return runtimeVersion;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationStatus#getPhase() phase} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for phase (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationStatus withPhase(@Nullable String value) {
    if (Objects.equals(this.phase, value)) return this;
    return new ImmutableIntegrationStatus(
        value,
        this.digest,
        this.image,
        this.dependencies,
        this.context,
        this.generatedSources,
        this.failure,
        this.camelVersion,
        this.runtimeVersion);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationStatus#getDigest() digest} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for digest (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationStatus withDigest(@Nullable String value) {
    if (Objects.equals(this.digest, value)) return this;
    return new ImmutableIntegrationStatus(
        this.phase,
        value,
        this.image,
        this.dependencies,
        this.context,
        this.generatedSources,
        this.failure,
        this.camelVersion,
        this.runtimeVersion);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationStatus#getImage() image} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for image (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationStatus withImage(@Nullable String value) {
    if (Objects.equals(this.image, value)) return this;
    return new ImmutableIntegrationStatus(
        this.phase,
        this.digest,
        value,
        this.dependencies,
        this.context,
        this.generatedSources,
        this.failure,
        this.camelVersion,
        this.runtimeVersion);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationStatus#getDependencies() dependencies}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationStatus withDependencies(String... elements) {
    ImmutableList<String> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationStatus(
        this.phase,
        this.digest,
        this.image,
        newValue,
        this.context,
        this.generatedSources,
        this.failure,
        this.camelVersion,
        this.runtimeVersion);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationStatus#getDependencies() dependencies}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of dependencies elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationStatus withDependencies(Iterable<String> elements) {
    if (this.dependencies == elements) return this;
    ImmutableList<String> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationStatus(
        this.phase,
        this.digest,
        this.image,
        newValue,
        this.context,
        this.generatedSources,
        this.failure,
        this.camelVersion,
        this.runtimeVersion);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationStatus#getContext() context} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for context (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationStatus withContext(@Nullable String value) {
    if (Objects.equals(this.context, value)) return this;
    return new ImmutableIntegrationStatus(
        this.phase,
        this.digest,
        this.image,
        this.dependencies,
        value,
        this.generatedSources,
        this.failure,
        this.camelVersion,
        this.runtimeVersion);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationStatus#getGeneratedSources() generatedSources}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationStatus withGeneratedSources(SourceSpec... elements) {
    ImmutableList<SourceSpec> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationStatus(
        this.phase,
        this.digest,
        this.image,
        this.dependencies,
        this.context,
        newValue,
        this.failure,
        this.camelVersion,
        this.runtimeVersion);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationStatus#getGeneratedSources() generatedSources}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of generatedSources elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationStatus withGeneratedSources(Iterable<? extends SourceSpec> elements) {
    if (this.generatedSources == elements) return this;
    ImmutableList<SourceSpec> newValue = ImmutableList.copyOf(elements);
    return new ImmutableIntegrationStatus(
        this.phase,
        this.digest,
        this.image,
        this.dependencies,
        this.context,
        newValue,
        this.failure,
        this.camelVersion,
        this.runtimeVersion);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationStatus#getFailure() failure} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for failure (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationStatus withFailure(@Nullable Failure value) {
    if (this.failure == value) return this;
    return new ImmutableIntegrationStatus(
        this.phase,
        this.digest,
        this.image,
        this.dependencies,
        this.context,
        this.generatedSources,
        value,
        this.camelVersion,
        this.runtimeVersion);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationStatus#getCamelVersion() camelVersion} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for camelVersion (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationStatus withCamelVersion(@Nullable String value) {
    if (Objects.equals(this.camelVersion, value)) return this;
    return new ImmutableIntegrationStatus(
        this.phase,
        this.digest,
        this.image,
        this.dependencies,
        this.context,
        this.generatedSources,
        this.failure,
        value,
        this.runtimeVersion);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationStatus#getRuntimeVersion() runtimeVersion} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for runtimeVersion (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationStatus withRuntimeVersion(@Nullable String value) {
    if (Objects.equals(this.runtimeVersion, value)) return this;
    return new ImmutableIntegrationStatus(
        this.phase,
        this.digest,
        this.image,
        this.dependencies,
        this.context,
        this.generatedSources,
        this.failure,
        this.camelVersion,
        value);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableIntegrationStatus} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableIntegrationStatus
        && equalTo((ImmutableIntegrationStatus) another);
  }

  private boolean equalTo(ImmutableIntegrationStatus another) {
    return Objects.equals(phase, another.phase)
        && Objects.equals(digest, another.digest)
        && Objects.equals(image, another.image)
        && dependencies.equals(another.dependencies)
        && Objects.equals(context, another.context)
        && generatedSources.equals(another.generatedSources)
        && Objects.equals(failure, another.failure)
        && Objects.equals(camelVersion, another.camelVersion)
        && Objects.equals(runtimeVersion, another.runtimeVersion);
  }

  /**
   * Computes a hash code from attributes: {@code phase}, {@code digest}, {@code image}, {@code dependencies}, {@code context}, {@code generatedSources}, {@code failure}, {@code camelVersion}, {@code runtimeVersion}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    @Var int h = 5381;
    h += (h << 5) + Objects.hashCode(phase);
    h += (h << 5) + Objects.hashCode(digest);
    h += (h << 5) + Objects.hashCode(image);
    h += (h << 5) + dependencies.hashCode();
    h += (h << 5) + Objects.hashCode(context);
    h += (h << 5) + generatedSources.hashCode();
    h += (h << 5) + Objects.hashCode(failure);
    h += (h << 5) + Objects.hashCode(camelVersion);
    h += (h << 5) + Objects.hashCode(runtimeVersion);
    return h;
  }

  /**
   * Prints the immutable value {@code IntegrationStatus} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("IntegrationStatus")
        .omitNullValues()
        .add("phase", phase)
        .add("digest", digest)
        .add("image", image)
        .add("dependencies", dependencies)
        .add("context", context)
        .add("generatedSources", generatedSources)
        .add("failure", failure)
        .add("camelVersion", camelVersion)
        .add("runtimeVersion", runtimeVersion)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link IntegrationStatus} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable IntegrationStatus instance
   */
  public static ImmutableIntegrationStatus copyOf(IntegrationStatus instance) {
    if (instance instanceof ImmutableIntegrationStatus) {
      return (ImmutableIntegrationStatus) instance;
    }
    return new IntegrationStatus.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableIntegrationStatus ImmutableIntegrationStatus}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "IntegrationStatus", generator = "Immutables")
  @NotThreadSafe
  @JsonInclude(JsonInclude.Include.NON_NULL)
  public static class Builder {
    private @Nullable String phase;
    private @Nullable String digest;
    private @Nullable String image;
    private ImmutableList.Builder<String> dependencies = ImmutableList.builder();
    private @Nullable String context;
    private ImmutableList.Builder<SourceSpec> generatedSources = ImmutableList.builder();
    private @Nullable Failure failure;
    private @Nullable String camelVersion;
    private @Nullable String runtimeVersion;

    /**
     * Creates a builder for {@link ImmutableIntegrationStatus ImmutableIntegrationStatus} instances.
     * <pre>
     * new IntegrationStatus.Builder()
     *    .phase(String | null) // nullable {@link IntegrationStatus#getPhase() phase}
     *    .digest(String | null) // nullable {@link IntegrationStatus#getDigest() digest}
     *    .image(String | null) // nullable {@link IntegrationStatus#getImage() image}
     *    .addDependencies|addAllDependencies(String) // {@link IntegrationStatus#getDependencies() dependencies} elements
     *    .context(String | null) // nullable {@link IntegrationStatus#getContext() context}
     *    .addGeneratedSources|addAllGeneratedSources(io.syndesis.server.controller.integration.camelk.crd.SourceSpec) // {@link IntegrationStatus#getGeneratedSources() generatedSources} elements
     *    .failure(io.syndesis.server.controller.integration.camelk.crd.Failure | null) // nullable {@link IntegrationStatus#getFailure() failure}
     *    .camelVersion(String | null) // nullable {@link IntegrationStatus#getCamelVersion() camelVersion}
     *    .runtimeVersion(String | null) // nullable {@link IntegrationStatus#getRuntimeVersion() runtimeVersion}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof IntegrationStatus.Builder)) {
        throw new UnsupportedOperationException("Use: new IntegrationStatus.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code IntegrationStatus} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationStatus.Builder from(IntegrationStatus instance) {
      Objects.requireNonNull(instance, "instance");
      @Nullable String phaseValue = instance.getPhase();
      if (phaseValue != null) {
        phase(phaseValue);
      }
      @Nullable String digestValue = instance.getDigest();
      if (digestValue != null) {
        digest(digestValue);
      }
      @Nullable String imageValue = instance.getImage();
      if (imageValue != null) {
        image(imageValue);
      }
      addAllDependencies(instance.getDependencies());
      @Nullable String contextValue = instance.getContext();
      if (contextValue != null) {
        context(contextValue);
      }
      addAllGeneratedSources(instance.getGeneratedSources());
      @Nullable Failure failureValue = instance.getFailure();
      if (failureValue != null) {
        failure(failureValue);
      }
      @Nullable String camelVersionValue = instance.getCamelVersion();
      if (camelVersionValue != null) {
        camelVersion(camelVersionValue);
      }
      @Nullable String runtimeVersionValue = instance.getRuntimeVersion();
      if (runtimeVersionValue != null) {
        runtimeVersion(runtimeVersionValue);
      }
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationStatus#getPhase() phase} attribute.
     * @param phase The value for phase (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("phase")
    public final IntegrationStatus.Builder phase(@Nullable String phase) {
      this.phase = phase;
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationStatus#getDigest() digest} attribute.
     * @param digest The value for digest (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("digest")
    public final IntegrationStatus.Builder digest(@Nullable String digest) {
      this.digest = digest;
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationStatus#getImage() image} attribute.
     * @param image The value for image (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("image")
    public final IntegrationStatus.Builder image(@Nullable String image) {
      this.image = image;
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationStatus#getDependencies() dependencies} list.
     * @param element A dependencies element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationStatus.Builder addDependencies(String element) {
      this.dependencies.add(element);
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationStatus#getDependencies() dependencies} list.
     * @param elements An array of dependencies elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationStatus.Builder addDependencies(String... elements) {
      this.dependencies.add(elements);
      return (IntegrationStatus.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link IntegrationStatus#getDependencies() dependencies} list.
     * @param elements An iterable of dependencies elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("dependencies")
    public final IntegrationStatus.Builder dependencies(Iterable<String> elements) {
      this.dependencies = ImmutableList.builder();
      return addAllDependencies(elements);
    }

    /**
     * Adds elements to {@link IntegrationStatus#getDependencies() dependencies} list.
     * @param elements An iterable of dependencies elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationStatus.Builder addAllDependencies(Iterable<String> elements) {
      this.dependencies.addAll(elements);
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationStatus#getContext() context} attribute.
     * @param context The value for context (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("context")
    public final IntegrationStatus.Builder context(@Nullable String context) {
      this.context = context;
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationStatus#getGeneratedSources() generatedSources} list.
     * @param element A generatedSources element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationStatus.Builder addGeneratedSources(SourceSpec element) {
      this.generatedSources.add(element);
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationStatus#getGeneratedSources() generatedSources} list.
     * @param elements An array of generatedSources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationStatus.Builder addGeneratedSources(SourceSpec... elements) {
      this.generatedSources.add(elements);
      return (IntegrationStatus.Builder) this;
    }


    /**
     * Sets or replaces all elements for {@link IntegrationStatus#getGeneratedSources() generatedSources} list.
     * @param elements An iterable of generatedSources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("generatedSources")
    public final IntegrationStatus.Builder generatedSources(Iterable<? extends SourceSpec> elements) {
      this.generatedSources = ImmutableList.builder();
      return addAllGeneratedSources(elements);
    }

    /**
     * Adds elements to {@link IntegrationStatus#getGeneratedSources() generatedSources} list.
     * @param elements An iterable of generatedSources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final IntegrationStatus.Builder addAllGeneratedSources(Iterable<? extends SourceSpec> elements) {
      this.generatedSources.addAll(elements);
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationStatus#getFailure() failure} attribute.
     * @param failure The value for failure (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("failure")
    public final IntegrationStatus.Builder failure(@Nullable Failure failure) {
      this.failure = failure;
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationStatus#getCamelVersion() camelVersion} attribute.
     * @param camelVersion The value for camelVersion (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("camelVersion")
    public final IntegrationStatus.Builder camelVersion(@Nullable String camelVersion) {
      this.camelVersion = camelVersion;
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationStatus#getRuntimeVersion() runtimeVersion} attribute.
     * @param runtimeVersion The value for runtimeVersion (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("runtimeVersion")
    public final IntegrationStatus.Builder runtimeVersion(@Nullable String runtimeVersion) {
      this.runtimeVersion = runtimeVersion;
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableIntegrationStatus ImmutableIntegrationStatus}.
     * @return An immutable instance of IntegrationStatus
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableIntegrationStatus build() {
      return new ImmutableIntegrationStatus(
          phase,
          digest,
          image,
          dependencies.build(),
          context,
          generatedSources.build(),
          failure,
          camelVersion,
          runtimeVersion);
    }
  }
}
