package io.syndesis.server.controller.integration.camelk.crd;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonUnwrapped;
import com.google.common.base.MoreObjects;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import com.google.errorprone.annotations.Var;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Objects;
import javax.annotation.CheckReturnValue;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link ResourceSpec}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new ResourceSpec.Builder()}.
 */
@Generated(from = "ResourceSpec", generator = "Immutables")
@SuppressWarnings({"ArrayEquals", "ArrayHashCode", "ArrayToString", "all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
@CheckReturnValue
public final class ImmutableResourceSpec
    implements ResourceSpec {
  private final @Nullable DataSpec dataSpec;
  private final @Nullable String type;
  private final @Nullable String mountPath;

  private ImmutableResourceSpec(
      @Nullable DataSpec dataSpec,
      @Nullable String type,
      @Nullable String mountPath) {
    this.dataSpec = dataSpec;
    this.type = type;
    this.mountPath = mountPath;
  }

  /**
   * @return The value of the {@code dataSpec} attribute
   */
  @JsonProperty("dataSpec")
  @JsonUnwrapped
  @Override
  public @Nullable DataSpec getDataSpec() {
    return dataSpec;
  }

  /**
   * @return The value of the {@code type} attribute
   */
  @JsonProperty("type")
  @Override
  public @Nullable String getType() {
    return type;
  }

  /**
   * @return The value of the {@code mountPath} attribute
   */
  @JsonProperty("mountPath")
  @Override
  public @Nullable String getMountPath() {
    return mountPath;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ResourceSpec#getDataSpec() dataSpec} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for dataSpec (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableResourceSpec withDataSpec(@Nullable DataSpec value) {
    if (this.dataSpec == value) return this;
    return new ImmutableResourceSpec(value, this.type, this.mountPath);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ResourceSpec#getType() type} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for type (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableResourceSpec withType(@Nullable String value) {
    if (Objects.equals(this.type, value)) return this;
    return new ImmutableResourceSpec(this.dataSpec, value, this.mountPath);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ResourceSpec#getMountPath() mountPath} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for mountPath (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableResourceSpec withMountPath(@Nullable String value) {
    if (Objects.equals(this.mountPath, value)) return this;
    return new ImmutableResourceSpec(this.dataSpec, this.type, value);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableResourceSpec} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableResourceSpec
        && equalTo((ImmutableResourceSpec) another);
  }

  private boolean equalTo(ImmutableResourceSpec another) {
    return Objects.equals(dataSpec, another.dataSpec)
        && Objects.equals(type, another.type)
        && Objects.equals(mountPath, another.mountPath);
  }

  /**
   * Computes a hash code from attributes: {@code dataSpec}, {@code type}, {@code mountPath}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    @Var int h = 5381;
    h += (h << 5) + Objects.hashCode(dataSpec);
    h += (h << 5) + Objects.hashCode(type);
    h += (h << 5) + Objects.hashCode(mountPath);
    return h;
  }

  /**
   * Prints the immutable value {@code ResourceSpec} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("ResourceSpec")
        .omitNullValues()
        .add("dataSpec", dataSpec)
        .add("type", type)
        .add("mountPath", mountPath)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link ResourceSpec} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable ResourceSpec instance
   */
  public static ImmutableResourceSpec copyOf(ResourceSpec instance) {
    if (instance instanceof ImmutableResourceSpec) {
      return (ImmutableResourceSpec) instance;
    }
    return new ResourceSpec.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableResourceSpec ImmutableResourceSpec}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "ResourceSpec", generator = "Immutables")
  @NotThreadSafe
  @JsonInclude(JsonInclude.Include.NON_NULL)
  public static class Builder {
    private @Nullable DataSpec dataSpec;
    private @Nullable String type;
    private @Nullable String mountPath;

    /**
     * Creates a builder for {@link ImmutableResourceSpec ImmutableResourceSpec} instances.
     * <pre>
     * new ResourceSpec.Builder()
     *    .dataSpec(io.syndesis.server.controller.integration.camelk.crd.DataSpec | null) // nullable {@link ResourceSpec#getDataSpec() dataSpec}
     *    .type(String | null) // nullable {@link ResourceSpec#getType() type}
     *    .mountPath(String | null) // nullable {@link ResourceSpec#getMountPath() mountPath}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof ResourceSpec.Builder)) {
        throw new UnsupportedOperationException("Use: new ResourceSpec.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code ResourceSpec} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final ResourceSpec.Builder from(ResourceSpec instance) {
      Objects.requireNonNull(instance, "instance");
      @Nullable DataSpec dataSpecValue = instance.getDataSpec();
      if (dataSpecValue != null) {
        dataSpec(dataSpecValue);
      }
      @Nullable String typeValue = instance.getType();
      if (typeValue != null) {
        type(typeValue);
      }
      @Nullable String mountPathValue = instance.getMountPath();
      if (mountPathValue != null) {
        mountPath(mountPathValue);
      }
      return (ResourceSpec.Builder) this;
    }

    /**
     * Initializes the value for the {@link ResourceSpec#getDataSpec() dataSpec} attribute.
     * @param dataSpec The value for dataSpec (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("dataSpec")
    @JsonUnwrapped
    public final ResourceSpec.Builder dataSpec(@Nullable DataSpec dataSpec) {
      this.dataSpec = dataSpec;
      return (ResourceSpec.Builder) this;
    }

    /**
     * Initializes the value for the {@link ResourceSpec#getType() type} attribute.
     * @param type The value for type (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("type")
    public final ResourceSpec.Builder type(@Nullable String type) {
      this.type = type;
      return (ResourceSpec.Builder) this;
    }

    /**
     * Initializes the value for the {@link ResourceSpec#getMountPath() mountPath} attribute.
     * @param mountPath The value for mountPath (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("mountPath")
    public final ResourceSpec.Builder mountPath(@Nullable String mountPath) {
      this.mountPath = mountPath;
      return (ResourceSpec.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableResourceSpec ImmutableResourceSpec}.
     * @return An immutable instance of ResourceSpec
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableResourceSpec build() {
      return new ImmutableResourceSpec(dataSpec, type, mountPath);
    }
  }
}
